@@ -358,7 +358,7 @@ class Primitive(PhonopyAtoms):
 
     def _set_smallest_vectors(self, supercell):
         self._smallest_vectors, self._multiplicity = _get_smallest_vectors(
-            supercell, self, self._symprec)
+            supercell, self, symprec=self._symprec)
 
     def _set_atomic_permutations(self, supercell):
         positions = supercell.get_scaled_positions()
@@ -469,13 +469,13 @@ def get_reduced_bases(lattice,
 
     Parameters
     ----------
-    lattice: ndarray or list of list
+    lattice : ndarray or list of list
         Basis vectors by row vectors, [a, b, c]^T
         shape=(3,3)
-    method: str
+    method : str
         delaunay: Delaunay reduction
         niggli: Niggli reduction
-    tolerance: float
+    tolerance : float
         Tolerance to find shortest basis vecotrs
 
     Returns
@@ -489,70 +489,24 @@ def get_reduced_bases(lattice,
     else:
         return spg.delaunay_reduce(lattice, eps=tolerance)
 
-#
-# Shortest pairs of atoms in supercell (Wigner-Seitz like)
-#
-# This is currently no longer used in phonopy, but still used by
-# phono3py. In phono3py, this is used to measure the shortest distance
-# between arbitrary pair of atoms in supercell. Therefore this method
-# may be moved to phono3py, but this way of use can also happen in
-# phonopy in the future, so let's keep it for a while.
-#
-def get_equivalent_smallest_vectors(atom_number_supercell,
-                                    atom_number_primitive,
-                                    supercell,
-                                    primitive_lattice,
-                                    symprec):
-    reduced_bases = get_reduced_bases(supercell.get_cell(), symprec)
-    reduced_bases_inv = np.linalg.inv(reduced_bases)
-    cart_positions = supercell.get_positions()
+def _get_smallest_vectors(supercell, primitive, symprec=1e-5):
+    p2s_map = primitive.get_primitive_to_supercell_map()
+    supercell_pos = supercell.get_scaled_positions()
+    primitive_pos = supercell_pos[p2s_map]
+    supercell_bases = supercell.get_cell()
+    primitive_bases = primitive.get_cell()
+    svecs, multi = get_smallest_vectors(
+        supercell_bases, supercell_pos, primitive_pos, symprec=symprec)
+    trans_mat_float = np.dot(supercell_bases, np.linalg.inv(primitive_bases))
+    trans_mat = np.rint(trans_mat_float).astype(int)
+    assert (np.abs(trans_mat_float - trans_mat) < 1e-8).all()
+    svecs = np.array(np.dot(svecs, trans_mat), dtype='double', order='C')
+    return svecs, multi
 
-    # Atomic positions are confined into the delaunay-reduced superlattice.
-    # Their positions will lie in the range -0.5 < x < 0.5, so that vectors
-    # drawn between them have components in the range -1 < x < 1.
-    def reduced_frac_pos(i):
-        vec = np.dot(cart_positions[i], reduced_bases_inv)
-        return vec - np.rint(vec)
-    p_pos = reduced_frac_pos(atom_number_primitive)
-    s_pos = reduced_frac_pos(atom_number_supercell)
-
-    # The vector arrow is from the atom in the primitive cell to the
-    # atom in the supercell.
-    differences = _get_equivalent_smallest_vectors_simple(s_pos - p_pos,
-                                                          reduced_bases,
-                                                          symprec)
-
-    # Return fractional coords in the basis of the primitive cell
-    #  rather than the supercell.
-    relative_scale = reduced_bases.dot(np.linalg.inv(primitive_lattice))
-    return differences.dot(relative_scale)
-
-# Given:
-#  - A delaunay-reduced lattice (row vectors)
-#  - A fractional vector (with respect to that lattice)
-#      whose coords lie in the range (-1 < x < 1)
-# Produce:
-#  - All fractional vectors of shortest length that are translationally
-#      equivalent to that vector under the lattice.
-def _get_equivalent_smallest_vectors_simple(frac_vector,
-                                            reduced_bases, # row vectors
-                                            symprec):
-
-    # Try all nearby images of the vector
-    lattice_points = np.array([
-        [i, j, k] for i in (-1, 0, 1)
-                  for j in (-1, 0, 1)
-                  for k in (-1, 0, 1)
-    ])
-    candidates = frac_vector + lattice_points
-
-    # Filter out the best ones by computing cartesian lengths.
-    # (A "clever" optimizer might try to skip the square root calculation here,
-    #  but he would be wrong; we're comparing a *difference* to the tolerance)
-    lengths = np.sqrt(np.sum(np.dot(candidates, reduced_bases)**2, axis=1))
-    return candidates[lengths - lengths.min() < symprec]
-
-def _get_smallest_vectors(supercell, primitive, symprec):
+def get_smallest_vectors(supercell_bases,
+                         supercell_pos,
+                         primitive_pos,
+                         symprec=1e-5):
     """Find shortest atomic pair vectors
 
     Note
@@ -562,44 +516,58 @@ def _get_smallest_vectors(supercell, primitive, symprec):
     atom in supercell is on the border centered at an atom in
     primitive and there are multiple vectors that have the same
     distance (up to tolerance) and different directions, several
-    shortest vectors are stored. The multiplicity is stored in another
-    array, "multiplicity".  (size_super, size_prim, multiple-vectors,
-    3)
+    shortest vectors are stored.
+    In fact, this method is not limited to search shortest vectors between
+    sueprcell atoms and primitive cell atoms, but can be used to measure
+    shortest vectors between atoms in periodic supercell lattice frame.
 
     Parameters
     ----------
-    supercell: Supercell
-        Supercell
-    primitive: Primitive
-        Primitive cell
-    symprec: float
+    supercell_bases : ndarray
+        Supercell basis vectors as row vectors, (a, b, c)^T. Must be order='C'.
+        dtype='double'
+        shape=(3, 3)
+    supercell_pos : array_like
+        Atomic positions in fractional coordinates of supercell.
+        dtype='double'
+        shape=(size_super, 3)
+    primitive_pos : array_like
+        Atomic positions in fractional coordinates of supercell. Note that not
+        in fractional coodinates of primitive cell.
+        dtype='double'
+        shape=(size_prim, 3)
+    symprec : float, optional, default=1e-5
         Tolerance to find equal distances of vectors
 
     Returns
     -------
-    tuple
-        (shortest_vectors, multiplicities)
-        shortest_vectors: ndarray
-            Shortest vectors (see Note). The 27 in shape is the possible
-            maximum number of elements.
-            shape=(size_super, size_prim, 27, 3)
-        multiplicities: ndarray
-            Number of equidistance shortest vectors
-            shape=(size_super, size_prim)
+    shortest_vectors : ndarray
+        Shortest vectors in supercell coordinates. The 27 in shape is the
+        possible maximum number of elements.
+        dtype='double'
+        shape=(size_super, size_prim, 27, 3)
+    multiplicities : ndarray
+        Number of equidistance shortest vectors
+        dtype='intc'
+        shape=(size_super, size_prim)
 
     """
 
-    # useful data from arguments
-    p2s_map = primitive.get_primitive_to_supercell_map()
-    size_super = supercell.get_number_of_atoms()
-    size_prim = primitive.get_number_of_atoms()
-    reduced_bases = get_reduced_bases(supercell.get_cell(), symprec)
+    reduced_bases = get_reduced_bases(supercell_bases,
+                                      method='delaunay',
+                                      tolerance=symprec)
+    trans_mat_float = np.dot(supercell_bases, np.linalg.inv(reduced_bases))
+    trans_mat = np.rint(trans_mat_float).astype(int)
+    assert (np.abs(trans_mat_float - trans_mat) < 1e-8).all()
+    trans_mat_inv_float = np.linalg.inv(trans_mat)
+    trans_mat_inv = np.rint(trans_mat_inv_float).astype(int)
+    assert (np.abs(trans_mat_inv_float - trans_mat_inv) < 1e-8).all()
 
     # Reduce all positions into the cell formed by the reduced bases.
-    supercell_fracs = np.dot(supercell.get_positions(),
-                             np.linalg.inv(reduced_bases))
+    supercell_fracs = np.dot(supercell_pos, trans_mat)
     supercell_fracs -= np.rint(supercell_fracs)
-    primitive_fracs = supercell_fracs[list(p2s_map)]
+    primitive_fracs = np.dot(primitive_pos, trans_mat)
+    primitive_fracs -= np.rint(primitive_fracs)
 
     # For each vector, we will need to consider all nearby images in the reduced bases.
     lattice_points = np.array([
@@ -645,9 +613,7 @@ def _get_smallest_vectors(supercell, primitive, symprec):
     # by the primitive cell.
     #
     # shape: (size_super, size_prim, 27, 3)
-    candidate_vectors = np.array(np.dot(
-        candidate_fracs,
-        reduced_bases.dot(np.linalg.inv(primitive.get_cell()))),
+    candidate_vectors = np.array(np.dot(candidate_fracs, trans_mat_inv),
                                  dtype='double', order='C')
 
     # The last final bits are done in C.
@@ -655,7 +621,7 @@ def _get_smallest_vectors(supercell, primitive, symprec):
     # We will gather the shortest ones from each list of 27 vectors.
     shortest_vectors = np.zeros_like(candidate_vectors,
                                      dtype='double', order='C')
-    multiplicity = np.zeros((size_super, size_prim), dtype='intc', order='C')
+    multiplicity = np.zeros(shortest_vectors.shape[:2], dtype='intc', order='C')
 
     import phonopy._phonopy as phonoc
     phonoc.gsv_copy_smallest_vectors(shortest_vectors,
